from django.contrib import admin
from ccgallery.models import Item, ItemImage, Category
from ccgallery.forms import ItemAdminForm, CategoryAdminForm

class CategoryAdmin(admin.ModelAdmin):
    form = CategoryAdminForm
    prepopulated_fields = {'slug': ('name',)}
    search_fields = (
            'name',
            'description',)
    list_filter = (
            'status',)
    list_display = (
            'name',
            'description',
            'status',)
    list_editable = (
            'status',)
    fieldsets = (
        (None, {
            'fields': ('name',
                        'status',
                        'description',
                        ),
            'classes': ('content',),
        }),
        ('Other Stuff', {
            'fields': ('slug',
                        'created'),
            'classes': ('collapse', 'other')
        })
    )


class ImageInline(admin.TabularInline):
    model = ItemImage

class ItemAdmin(admin.ModelAdmin):
    inlines = [ImageInline]
    date_hierarchy = 'created'
    prepopulated_fields = {'slug': ('name',)}
    form = ItemAdminForm
    filter_horizontal = (
            'categories',)
    search_fields = (
            'name',
            'description',)
    list_filter = (
            'status',)
    list_editable = (
            'status',)
    list_display = (
            'name',
            'description',
            'status')
    fieldsets = (
        (None, {
            'fields': ('name',
                        'status',
                        'description',
                        'categories'),
            'classes': ('content',),
        }),
        ('Other Stuff', {
            'fields': ('slug',
                        'created'),
            'classes': ('collapse', 'other')
        })
    )

admin.site.register(Item, ItemAdmin)
admin.site.register(Category, CategoryAdmin)



from django.conf import settings


"""The default status for items"""
ITEM_DEFAULT_STATUS = getattr(
        settings,
        'CCGALLERY_ITEM_DEFAULT_STATUS',
        1)

"""The default status for categories"""
CATEGORY_DEFAULT_STATUS = getattr(
        settings,
        'CCGALLERY_CATEGORY_DEFAULT_STATUS',
        1)

"""If false then the bundled model will be used"""
MODEL = getattr(
        settings,
        'CCGALLERY_MODEL',
        False)

"""The image sizes for the gallery item"""
IMAGE_SIZES = getattr(
        settings,
        'CCGALLERY_IMAGE_SIZES',
        (   (200,200),
            (640, 480),
            (1200, 960)))

"""The markup language to use"""
MARKUP_LANGUAGE = getattr(
        settings,
        'CCGALLERY_MARKUP_LANGUAGE',
        'markdown')



try:
    import textile
except ImportError:
    pass
try:
    import markdown
except ImportError:
    pass
from django import template
from django.conf import settings
from django.utils.safestring import mark_safe
from ccgallery import settings as c_settings
from ccgallery.models import Item

register = template.Library()

class ItemNode(template.Node):

    def __init__(self, display, varname):
        self.display = display
        self.varname = varname

    def render(self, context):
        items = Item.objects\
                .visible()\
                .order_by('?')[:self.display]
        context[self.varname] = items
        return ''

@register.tag
def get_gallery_items(parser, token):
    bits = token.contents.split()
    return ItemNode(bits[1], bits[3])

@register.inclusion_tag('ccgallery/_js.html')
def gallery_js():
    return {
        'STATIC_URL': settings.STATIC_URL,
    }

@register.inclusion_tag('ccgallery/_css.html')
def gallery_css():
    return {
        'STATIC_URL': settings.STATIC_URL,
    }

@register.filter
def markup(text):
    """output the description according to whatever markup
    language is set in the settings"""
    html = ''
    if c_settings.MARKUP_LANGUAGE == 'textile':
        html = textile.textile(text)

    if c_settings.MARKUP_LANGUAGE == 'markdown':
        html = markdown.markdown(text)

    return mark_safe(html)



import os
from decimal import Decimal
from unittest import skipUnless
from django.test import TestCase
from django.conf import settings
from django.core.files import File
from ccgallery.models import Item, Category, ItemImage


red = os.path.exists('%s/ccgallery/red.jpg' % settings.STATIC_ROOT)
blue = os.path.exists('%s/ccgallery/blue.jpg' % settings.STATIC_ROOT)

class CategoryTestCases(TestCase):

    @skipUnless(red, 'red.png file does not exist')
    @skipUnless(blue, 'blue.png file does not exist')
    def test_category_images(self):
        """ A category will return the first image from the first 
        available item in it's set"""
        # the images
        red = open('%s/ccgallery/red.jpg' % settings.STATIC_ROOT)
        blue = open('%s/ccgallery/blue.jpg' % settings.STATIC_ROOT)
        # make items
        i1 = Item()
        i1.status = Item.VISIBLE
        i1.name = '1'
        i1.slug = '1'
        i1.title = '1'
        i1.description = '1'
        i1.save()
        i2 = Item()
        i2.status = Item.VISIBLE
        i2.name = '2'
        i2.slug = '2'
        i2.title = '2'
        i2.description = '2'
        i2.order = Decimal('2.00')
        i2.save()
        # make the images
        ii1 = ItemImage()
        ii1.item = i1
        ii1.src = File(red, 'ccgallery/red.jpg')
        ii1.order = Decimal('10.00')
        ii1.save()
        ii2 = ItemImage()
        ii2.item = i2
        ii2.src = File(blue, 'ccgallery/blue.jpg')
        ii2.save()
        # make category
        c1 = Category()
        c1.slug = '1'
        c1.title = '1'
        c1.description = '1'
        c1.status = Category.VISIBLE
        c1.save()
        # add images to the categories
        i1.categories.add(c1)
        i2.categories.add(c1)
        # close the images
        red.close()
        blue.close()
        # now the category image is ii2
        self.assertEqual(c1.image.pk, ii2.pk)
        # make i2 hidden
        i2.status = Item.HIDDEN
        i2.save()
        # now the category image is ii1
        self.assertEqual(c1.image.pk, ii1.pk)
        # make i1 hidden and now it's none
        i1.status = Item.HIDDEN
        i1.save()
        # now the category image is ii1
        self.assertEqual(c1.image, None)


class ItemTestCases(TestCase):

    def test_ordering(self):
        """items are ordered by their order"""
        i1 = Item()
        i1.slug = '1'
        i1.title = '1'
        i1.description = '1'
        i1.save()
        i2 = Item()
        i2.slug = '2'
        i2.title = '2'
        i2.description = '2'
        i2.order = Decimal('2.00')
        i2.save()
        i3 = Item()
        i3.slug = '3'
        i3.title = '3'
        i3.description = '3'
        i3.order = Decimal('1.00')
        i3.save()
        # 3 is first, 2 second and 1 third
        self.assertEqual('3', Item.objects.all()[0].slug)
        self.assertEqual('2', Item.objects.all()[1].slug)
        self.assertEqual('1', Item.objects.all()[2].slug)


    def test_created(self):
        """items have a created date added when they're created"""
        i = Item()
        i.slug = '1'
        i.title = '1'
        i.description = '1'
        i.save()
        self.assertTrue(i.created)

    def test_url(self):
        """items return their url correctly"""
        i = Item()
        i.slug = '1'
        i.title = '1'
        i.description = '1'
        i.save()
        self.assertEqual('/gallery/1.html', i.get_absolute_url())



from django.test import TestCase
from django.conf import settings
from ccgallery import settings as c_settings
from ccgallery.models import get_model


class FakeModel(object):
    """Does nothing. Used in the testcases"""


class SettingTestCases(TestCase):

    def setUp(self):
        reload(c_settings)

    def tearDown(self):
        try:
            del(settings.CCGALLERY_MODEL)
        except AttributeError:
            pass

    def test_nonsense_model(self):
        """ if a custom model doesn't exists we get the
        bundled model returned"""
        # set thesettings.CCCONTACT_MODEL custom model
        settings.CCGALLERY_MODEL = 'nonsense.JabberWocky'
        reload(c_settings)
        self.assertEqual(
                get_model()._meta.module_name,
                'item')
        self.assertEqual(
                get_model()._meta.app_label,
                'ccgallery')

    def test_default_model(self):
        """The cccontact.model.Contact models is the default model"""
        self.assertEqual(
                get_model()._meta.module_name,
                'item')
        self.assertEqual(
                get_model()._meta.app_label,
                'ccgallery')

    def test_custom_model(self):
        """The custom model can be overridden"""
        # set thesettings.CCCONTACT_MODEL custom model
        settings.CCGALLERY_MODEL = 'ccgallery.tests.test_settings.FakeModel'
        reload(c_settings)
        # we have the fake model
        self.assertEqual('FakeModel', get_model().__name__)



from django.test import TestCase
from django.core.urlresolvers import reverse
from ccgallery.tests.mock import MockRequest
from ccgallery.models import Item, Category
from ccgallery.views import index, item, category

class ViewTestCases(TestCase):

    def setUp(self):
        self.rf = MockRequest()

    def test_category_404(self):
        """if a category doesn't exist a 404 is raised"""
        r = self.client.get(reverse('ccgallery:category', args=['1']))
        self.assertEqual(404, r.status_code)
    
    def test_item_404(self):
        """if a item doesn't exist a 404 is raised"""
        r = self.client.get(reverse('ccgallery:item', args=['1']))
        self.assertEqual(404, r.status_code)

    def test_index_200(self):
        """index responds with a 200 OK"""
        request = self.rf.get(reverse('ccgallery:index'))
        response  = index(request)
        # is 200
        self.assertEqual(200, response.status_code)

    def test_item_200(self):
        """item responds with a 200 OK"""
        i1 = Item()
        i1.slug = '1'
        i1.title = '1'
        i1.description = '1'
        i1.save()
        request = self.rf.get(reverse('ccgallery:item', args=[i1.slug]))
        response  = item(request, i1.slug)
        # is 200
        self.assertEqual(200, response.status_code)

    def test_item_200_with_category(self):
        """item responds with a 200 OK"""
        i1 = Item()
        i1.slug = '1'
        i1.title = '1'
        i1.description = '1'
        i1.save()
        c1 = Category()
        c1.slug = '1'
        c1.title = '1'
        c1.description = '1'
        c1.status = Category.VISIBLE
        c1.save()
        i1.categories.add(c1)
        request = self.rf.get(
                reverse('ccgallery:item', args=[c1.slug, i1.slug]))
        response  = item(request, i1.slug, c1.slug)
        # is 200
        self.assertEqual(200, response.status_code)

    def test_item_200_with_fake_category(self):
        """item responds with a 200 OK"""
        i1 = Item()
        i1.slug = '1'
        i1.title = '1'
        i1.description = '1'
        i1.save()
        request = self.rf.get(
                reverse('ccgallery:item', args=['horse', i1.slug]))
        response  = item(request, i1.slug, 'horse')
        # is 200
        self.assertEqual(200, response.status_code)
    def test_category_200(self):
        """category responds with a 200 OK"""
        c1 = Category()
        c1.slug = '1'
        c1.title = '1'
        c1.description = '1'
        c1.status = Category.VISIBLE
        c1.save()
        i1 = Item()
        i1.slug = '1'
        i1.title = '1'
        i1.description = '1'
        i1.save()
        i1.categories.add(c1)
        request = self.rf.get(
                reverse('ccgallery:category', args=[c1.slug]))
        response = category(request, c1.slug)
        # is 200
        self.assertEqual(200, response.status_code)



from django.db import models


class CategoryManager(models.Manager):

    def visible(self):
        return super(CategoryManager, self)\
                .get_query_set()\
                .filter(status=self.model.VISIBLE)

    def hidden(self):
        return super(CategoryManager, self)\
                .get_query_set()\
                .filter(status=self.model.HIDDEN)


class ItemManager(models.Manager):

    def for_category(self, category):
        return super(ItemManager, self)\
                .get_query_set()\
                .filter(categories__slug=category.slug,
                       categories__status=category.VISIBLE,
                       status=self.model.VISIBLE)

    def visible(self):
        return super(ItemManager, self)\
                .get_query_set()\
                .filter(status=self.model.VISIBLE)

    def hidden(self):
        return super(ItemManager, self)\
                .get_query_set()\
                .filter(status=self.model.HIDDEN)



from django.db import models
from ccgallery import import_thing
from ccgallery import listeners
from ccgallery import managers
import ccgallery.settings as c_settings
from ccthumbs.fields import ImageWithThumbsField


def get_model():
    model = Item
    if c_settings.MODEL:
        try:
            module, model = import_thing(c_settings.MODEL)
        except (AttributeError, ImportError):
            pass

    return model


class Category(models.Model):
    HIDDEN = 0
    VISIBLE = 1
    CATEGORY_CHOICES = (
        (HIDDEN, 'Hidden'),
        (VISIBLE, 'Visible'),
    )
    name = models.CharField(
            max_length=255)
    slug = models.SlugField()
    description = models.TextField(
            blank=True,
            null=True)
    order = models.DecimalField(
            max_digits=6,
            decimal_places=2,
            default='5.00')
    created = models.DateTimeField(
            blank=True,
            null=True)
    status = models.PositiveSmallIntegerField(
            choices=CATEGORY_CHOICES,
            default=c_settings.CATEGORY_DEFAULT_STATUS)

    objects = managers.CategoryManager()

    class Meta:
        ordering = ['order']

    def __unicode__(self):
        return u'%s' % self.name

    @models.permalink
    def get_absolute_url(self):
        return ('ccgallery:category', (), {
            'slug': self.slug,
            })

    @property
    def image(self):
        """returns the first image from the first visible item"""
        model_cls = get_model()
        try:
            items = self.item_set.filter(status=model_cls.VISIBLE)
            item = items[0]
            image = item.itemimage_set.all()[0]
        except IndexError:
            image = None
        return image

class Item(models.Model):
    HIDDEN = 0
    VISIBLE = 1
    ITEM_CHOICES = (
        (HIDDEN, 'Hidden'),
        (VISIBLE, 'Visible'),
    )
    name = models.CharField(
            max_length=255)
    slug = models.SlugField()
    categories = models.ManyToManyField(
            Category,
            blank=True,
            null=True)
    description = models.TextField(
            blank=True,
            null=True)
    order = models.DecimalField(
            max_digits=6,
            decimal_places=2,
            default='5.00')
    created = models.DateTimeField(
            blank=True,
            null=True)
    status = models.PositiveSmallIntegerField(
            choices=ITEM_CHOICES,
            default=c_settings.ITEM_DEFAULT_STATUS)

    objects = managers.ItemManager()

    class Meta:
        ordering = ['order']

    def __unicode__(self):
        return u'%s' % self.name

    @models.permalink
    def get_absolute_url(self):
        return ('ccgallery:item', (), {
            'slug': self.slug,
            })

    @property
    def image(self):
        """returns the first image from the first visible item"""
        model_cls = get_model()
        try:
            image = self.itemimage_set.all()[0]
        except IndexError:
            image = None
        return image


# TODO: test the ordering
class ItemImage(models.Model):
    item = models.ForeignKey(
            Item)
    caption = models.CharField(
            max_length=255,
            blank=True,
            null=True)
    src = ImageWithThumbsField(
                upload_to='ccgallery/itemimages/%Y/%m/%d',
                sizes=c_settings.IMAGE_SIZES)
    order = models.DecimalField(
            max_digits=6,
            decimal_places=2,
            default='5.00')

    class Meta:
        ordering = ['order']

    def __unicode__(self):
        return self.src.url

models.signals.pre_save.connect(
        listeners.created,
        sender=Item,
        dispatch_uid='item_created')



from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from ccgallery.models import Category, get_model



def category(request, slug):
    try:
        category = Category.objects.visible().get(slug=slug)
    except Category.DoesNotExist:
        raise Http404('Category slug not found')

    items = get_model().objects\
            .visible()\
            .filter(categories=category)

    return render_to_response(
            'ccgallery/category.html', {
                'category': category,
                'items': items,
                }, RequestContext(request))

def item(request, slug, category_slug=None):
    model_cls = get_model()
    category = None
    try:
        item = model_cls.objects.visible().get(slug=slug)
    except model_cls.DoesNotExist:
        raise Http404('Item slug not found')

    if category_slug != None:
        try:
            category = Category.objects.visible().get(
                    slug=category_slug)
        except Category.DoesNotExist:
            pass

    return render_to_response(
            'ccgallery/item.html', {
                'item': item,
                'category': category,
                }, RequestContext(request))


def index(request):
    categories = Category.objects.visible()

    return render_to_response(
            'ccgallery/index.html', {
                'categories': categories,
                }, RequestContext(request))



from django import forms
from ccgallery.models import Item, Category

class CategoryAdminForm(forms.ModelForm):

    class Meta:
        model = Category

    class Media:
        css = {
                'screen': ('ccgallery/css/admin.category.css',)
            }

class ItemAdminForm(forms.ModelForm):

    class Meta:
        model = Item

    class Media:
        css = {
                'screen': ('ccgallery/css/admin.item.css',)
            }



from django.conf.urls import patterns, include, url

urlpatterns = patterns('',
    url(r'^/(?P<slug>[\w\-]+).html',
        'ccgallery.views.item',
        name='item'),
    url(r'^/(?P<category_slug>[\w\-]+)/(?P<slug>[\w\-]+).html',
        'ccgallery.views.item',
        name='item'),
    url(r'^/(?P<slug>[\w\-]+)/',
        'ccgallery.views.category',
        name='category'),
    url(r'^\.html',
        'ccgallery.views.index',
        name='index'),
)



import os
from distutils.core import setup
from ccgallery import VERSION

root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)

setup(
    name='django-ccgallery',
    version=VERSION,
    license='BSD 3 Clause',
    description='A minimal Gallery application for Django',
    long_description=open('README.rst').read(),
    author='c&c Design Consultants LTD',
    author_email='studio@designcc.co.uk',
    url='https://github.com/designcc/django-ccgallery',
    package_data={
        'ccgallery' : [
            'templates/ccgallery/*.html',
            'static/ccgallery/*.jpg',
            'static/ccgallery/css/*.css',
            'static/ccgallery/fancybox/source/*.gif',
            'static/ccgallery/fancybox/source/*.png',
            'static/ccgallery/fancybox/source/*.css',
            'static/ccgallery/fancybox/source/*.js',
            'static/ccgallery/fancybox/source/*.png',
            'static/ccgallery/fancybox/source/helpers/*.png',
            'static/ccgallery/fancybox/source/helpers/*.css',
            'static/ccgallery/fancybox/source/helpers/*.js',
        ],
    },
    packages=[
        'ccgallery',
        'ccgallery.templatetags',
        'ccgallery.tests'
    ],
    install_requires=['django-ccthumbs',])



from decimal import Decimal
from django.test import TestCase
from ccgallery.models import Item, Category


class ManagerTestCases(TestCase):

    def test_item_for_category(self):
        """the for item manager method"""
        c1 = Category()
        c1.slug = '1'
        c1.title = '1'
        c1.description = '1'
        c1.status = Category.VISIBLE
        c1.save()
        # make the items
        i1 = Item()
        i1.slug = '1'
        i1.title = '1'
        i1.description = '1'
        i1.status = Item.VISIBLE
        i1.save()
        i1.categories.add(c1)
        i2 = Item()
        i2.slug = '2'
        i2.title = '2'
        i2.description = '2'
        i2.order = Decimal('2.00')
        i2.status = Item.VISIBLE
        i2.save()
        i2.categories.add(c1)
        # visible returns 1
        self.assertEqual(2, Item.objects.for_category(c1).count())
        # hidden returns 1
        i2.status = Item.HIDDEN
        i2.save()
        self.assertEqual(1, Item.objects.for_category(c1).count())
        # make the category invisible
        c1.status = Category.HIDDEN
        c1.save()
        self.assertEqual(0, Item.objects.for_category(c1).count())

    def test_category_visible(self):
        """only visible categorys are returned"""
        i1 = Category()
        i1.slug = '1'
        i1.title = '1'
        i1.description = '1'
        i1.status = Category.VISIBLE
        i1.save()
        i2 = Category()
        i2.slug = '2'
        i2.title = '2'
        i2.description = '2'
        i2.order = Decimal('2.00')
        i2.status = Category.VISIBLE
        i2.save()
        # visible returns 1
        self.assertEqual(2, Category.objects.visible().count())
        # hidden returns 1
        i2.status = Category.HIDDEN
        i2.save()
        self.assertEqual(1, Category.objects.visible().count())


    def test_category_hidden(self):
        """only hidden categorys are returned"""
        i1 = Category()
        i1.slug = '1'
        i1.title = '1'
        i1.description = '1'
        i1.status = Category.VISIBLE
        i1.save()
        i2 = Category()
        i2.slug = '2'
        i2.title = '2'
        i2.description = '2'
        i2.order = Decimal('2.00')
        i2.status = Category.VISIBLE
        i2.save()
        # visible returns 1
        self.assertEqual(0, Category.objects.hidden().count())
        # hidden returns 1
        i2.status = Category.HIDDEN
        i2.save()
        self.assertEqual(1, Category.objects.hidden().count())


    def test_item_visible(self):
        """only visible items are returned"""
        i1 = Item()
        i1.slug = '1'
        i1.title = '1'
        i1.description = '1'
        i1.status = Item.VISIBLE
        i1.save()
        i2 = Item()
        i2.slug = '2'
        i2.title = '2'
        i2.description = '2'
        i2.order = Decimal('2.00')
        i2.status = Item.VISIBLE
        i2.save()
        # visible returns 1
        self.assertEqual(2, Item.objects.visible().count())
        # hidden returns 1
        i2.status = Item.HIDDEN
        i2.save()
        self.assertEqual(1, Item.objects.visible().count())


    def test_item_hidden(self):
        """only hidden items are returned"""
        i1 = Item()
        i1.slug = '1'
        i1.title = '1'
        i1.description = '1'
        i1.status = Item.VISIBLE
        i1.save()
        i2 = Item()
        i2.slug = '2'
        i2.title = '2'
        i2.description = '2'
        i2.order = Decimal('2.00')
        i2.status = Item.VISIBLE
        i2.save()
        # visible returns 1
        self.assertEqual(0, Item.objects.hidden().count())
        # hidden returns 1
        i2.status = Item.HIDDEN
        i2.save()
        self.assertEqual(1, Item.objects.hidden().count())

